@@ -5,7 +5,8 @@ read.alignment <- function(file, format, forceToLower = TRUE)
 {
   #
   # Check that we have read permission on the file:
-  #
+  # 
+  file <- path.expand(file) 
   if(file.access(file, mode = 4) != 0) stop(paste("File", file, "is not readable"))
   
   ali <- switch( format,
