@@ -1,10 +1,6 @@
 #include "alignment.h"
 
 
-
-
-
-
 void rem_blank(char *string)
 {
   int ii;
@@ -101,7 +97,7 @@ SEXP read_mase(SEXP nomfic)
  
 
   /*Passages des objets R (paramtres) dans des variables C */ 
-  fic_name=CHAR(STRING_ELT(nomfic,0)); 
+  fic_name = (char *) CHAR(STRING_ELT(nomfic, 0)); 
 
 
 
@@ -109,7 +105,6 @@ SEXP read_mase(SEXP nomfic)
     error("Can't open file");
   }
   
-  
   c1 = 0;
   nb_seq = 0;
   lg = lgc = 0;
@@ -119,8 +114,8 @@ SEXP read_mase(SEXP nomfic)
     lgs = strlen(string);
     
     if(lgs >= (MAXSTRING - 1)) {
-      fprintf(stderr, "\n Too long line in alignment (> %d).\n", MAXSTRING);
-      error("increase MAXSTRING and recompile.\n"); 
+      REprintf("\n Fatal Error. Too long line in alignment (> %d).\n", MAXSTRING);
+      REprintf("Increase MAXSTRING and recompile.\n"); 
     }
     
     c2 = string[0];
@@ -173,9 +168,14 @@ SEXP read_mase(SEXP nomfic)
  while(fgets(string, MAXSTRING, fic) != NULL) {
    numline++;
    string[MAXSTRING] = 0;
-   
+   if ((string[0] != ';') && (numline == 1))
+   	{
+	error("Not a MASE file"); /* check format, thanks to J.H. Troesemeier */
+	goto fini;
+	}
    
    c2 = string[0];
+   
    if(string[0] == ';' && string[1] != ';') {
      strcat(aln[ii + 1].com, string);
         }
@@ -187,9 +187,10 @@ SEXP read_mase(SEXP nomfic)
      rem_blank(string);
 
      if((int) strlen(string) >= (MAXMNMASE - 1)) {
-       fprintf(stderr, "max  %d characters\n", MAXMNMASE);
+       REprintf("Error. Maximum sequance name is   %d characters\n", MAXMNMASE);
        error("sequence name too long!");
-     }
+     } 
+     
      strcpy(aln[ii].mn, string);
      
      lg = 0;
@@ -237,7 +238,7 @@ for(i=0;i<nb_seq;i++){
  SET_ELEMENT(essai,2,listseq);
  SET_ELEMENT(essai,3,listcom);
  
-  
+ fini: 
  free_mase(aln,nb_seq);
  UNPROTECT(5);
  
@@ -256,11 +257,12 @@ for(i=0;i<nb_seq;i++){
 /*************************************************************************/
 
 
-SEXP distance(SEXP sequences,SEXP nbseq, SEXP matNumber, SEXP seqtype){
+SEXP distance(SEXP sequences,SEXP nbseq, SEXP matNumber, SEXP seqtype, SEXP gapoption){
 
   SEXP d;
   int MAXNSEQS;
   char **seq;
+  int gap_option;
   int i, j, k, n,totseqs, seq_long, nbases;
   int mat_number, seq_type;
   int **ndiff;
@@ -298,13 +300,14 @@ SEXP distance(SEXP sequences,SEXP nbseq, SEXP matNumber, SEXP seqtype){
   totseqs = INTEGER_VALUE(nbseq);
   mat_number= INTEGER_VALUE(matNumber);
   seq_type = INTEGER_VALUE(seqtype);
+  gap_option =  INTEGER_VALUE(gapoption);
 
   PROTECT(d=NEW_NUMERIC(totseqs*totseqs));
 
   seq = (char **)malloc(totseqs*sizeof(char *));
    
   for(i=0;i<totseqs;i++){
-    seq[i] = CHAR(STRING_ELT(sequences,i));
+    seq[i] = (char *) CHAR(STRING_ELT(sequences,i));
   }
 
   ndiff = (int **)malloc(MAXNSEQS*sizeof(int *));
@@ -358,6 +361,23 @@ SEXP distance(SEXP sequences,SEXP nbseq, SEXP matNumber, SEXP seqtype){
 			ndiff[i][j]++;
 		      }
 		  }
+		  
+		  if (gap_option == 1)
+		  	{
+			if (seq[i][k] == '-'  && seq[j][k] != '-' && seq[j][k] != 'N' && seq[j][k] != 'X')
+				{
+				nbases++;
+				ndiff[j][i]++;
+				ndiff[i][j]++;
+				}
+			if (seq[j][k] == '-'  && seq[i][k] != '-' && seq[i][k] != 'N' && seq[i][k] != 'X')
+				{
+				nbases++;
+				ndiff[j][i]++;
+				ndiff[i][j]++;
+				}
+			
+			}
 	      }
 
 	      else if(mat_number == 1 && seq_type == 0){
@@ -440,7 +460,7 @@ SEXP read_msf_align(SEXP ficname)
  int i,l, curr_spec, maxwidname=0, curr_len, tot_spec, wid_1_line, wid_block;
  char **seq, **seqname, **comments;
  
- fname=CHAR(STRING_ELT(ficname,0)); 
+ fname = (char *) CHAR(STRING_ELT(ficname,0)); 
 
  PROTECT(nombreseq=NEW_INTEGER(1));
   PROTECT(list=allocVector(VECSXP,3));
@@ -480,7 +500,9 @@ SEXP read_msf_align(SEXP ficname)
    }
    tot_spec = -1;
    do	{
-     fgets(line,sizeof(line),in);
+     if(!fgets(line,sizeof(line),in))
+       break; 
+
      if( (p = strstr(line, "Name:") ) == NULL) continue;
      tot_spec++;
      q = strstr(p, " Len: "); 
@@ -560,7 +582,7 @@ SEXP read_phylip_align(SEXP ficname)
   int totseqs, lenseqs, i, l;
   
   q=0;
-  fname=CHAR(STRING_ELT(ficname,0)); 
+  fname = (char *) CHAR(STRING_ELT(ficname,0)); 
   
   PROTECT(nombreseq=NEW_INTEGER(1));
   PROTECT(list=allocVector(VECSXP,3));
@@ -570,8 +592,9 @@ SEXP read_phylip_align(SEXP ficname)
   if(in==NULL) {
     error("file not found");
   }
-  fgets(line,sizeof(line),in);
-  if( sscanf(line, "%d%d", &totseqs, &lenseqs) != 2) {
+  
+  if(!fgets(line,sizeof(line),in) ||
+     sscanf(line, "%d%d", &totseqs, &lenseqs) != 2) {
     error("Not a PHYLIP file");
     totseqs = 0;
     goto fini;
@@ -594,7 +617,7 @@ SEXP read_phylip_align(SEXP ficname)
     comments[i] = NULL;
   }
   for(i=0; i<totseqs; i++) {
-    fgets(line,sizeof(line),in);
+    if(!fgets(line,sizeof(line),in)) goto eof;
     memcpy(seqname[i],line,PHYNAME); seqname[i][PHYNAME] = 0;
     p = line+PHYNAME; q = seq[i];
     while(*p != '\n') {
@@ -604,9 +627,9 @@ SEXP read_phylip_align(SEXP ficname)
   }
   l = q - seq[totseqs - 1];
   while( l < lenseqs) {
-    fgets(line,sizeof(line),in);
+    if(!fgets(line,sizeof(line),in)) goto eof;
     for(i=0; i<totseqs; i++) {
-      fgets(line,sizeof(line),in);
+      if(!fgets(line,sizeof(line),in)) goto eof;
       p = line; q = seq[i] + l;
       while(*p != '\n') {
 	if(*p != ' ') *(q++) = *p;
@@ -637,18 +660,20 @@ SEXP read_phylip_align(SEXP ficname)
  error("Not enough memory!");
  totseqs = 0;
  goto fini;
+ eof:
+ error("Unexpected EOF/Corrupt File");
+ totseqs = 0;
+ goto fini;
 }
 
 
 
-/****************************************/
-/* Lecture  d'un fichier au format fasta */
-/***************************************/
-
+/*************************************/
+/* Reading alignment in fasta format */
+/*************************************/
 
 SEXP read_fasta_align(SEXP ficname)
 {
- 
   SEXP list;
   SEXP listseq;
   SEXP listname;
@@ -656,94 +681,88 @@ SEXP read_fasta_align(SEXP ficname)
   char *fname;
   FILE *in;
   int totseqs, lseq, l2, l, lenseqs;
-  char line[200], *p, *i, *provseq = NULL;
+  char line[200], *p, *i;
   char **seq, **seqname, **comments;
 
-  fname=CHAR(STRING_ELT(ficname,0)); 
+  fname = (char *) CHAR(STRING_ELT(ficname, 0)); 
 
-  PROTECT(nombreseq=NEW_INTEGER(1));
-  PROTECT(list=allocVector(VECSXP,3));
+  PROTECT(nombreseq = NEW_INTEGER(1));
+  PROTECT(list = allocVector(VECSXP, 3));
 
-  if( (in=fopen(fname,"r")) == NULL) {
-    error("file not found");
-	}
+  /* Check that the file is available */
 
-  /* calcul du nombre de sequences dans le fichier */
-  totseqs = 0;
-  while(fgets(line, sizeof(line), in) != NULL) {
-    if(*line == '>') totseqs++;
-  }
+  if((in = fopen(fname, "r")) == NULL) error("File not found");
   
-  INTEGER(nombreseq)[0]=totseqs;
-  PROTECT(listname=allocVector(VECSXP,totseqs));
-  PROTECT(listseq=allocVector(VECSXP,totseqs));
+  /* How many sequences are in the file ? */
 
- rewind(in);
- seq = (char **)malloc(totseqs * sizeof(char *));
- if(seq == NULL) goto nomem;
- comments = (char **)malloc(totseqs * sizeof(char *));
- if(comments == NULL) goto nomem;
- seqname = (char **)malloc(totseqs * sizeof(char *));
- if(seqname == NULL) goto nomem;
+  totseqs = 0;
+  while(fgets(line, sizeof(line), in) != NULL)
+    if(*line == '>') totseqs++;
+  rewind(in);
+
+  /* R objects creation */  
+  
+  INTEGER(nombreseq)[0] = totseqs;
+  PROTECT(listname = allocVector(VECSXP, totseqs));
+  PROTECT(listseq = allocVector(VECSXP, totseqs));
+
+  /* Memory allocation */
+
+  seq = (char **) R_alloc(totseqs, sizeof(char *));
+  comments = (char **) R_alloc(totseqs, sizeof(char *));
+  seqname = (char **) R_alloc(totseqs, sizeof(char *));
  
+  lenseqs = MAXLENSEQ;
+  totseqs = -1;
+  i = fgets(line, sizeof(line), in);
+  if(line[0] != '>')
+    error("File not in Fasta format!\n");
 
- lenseqs = MAXLENSEQ;
- totseqs = -1;
- i = fgets(line, sizeof(line), in);
- if(line[0] != '>') {
-   printf("File not in Fasta format!");
-   totseqs = -1; goto fini;
- }
- while( i != NULL ){
-   totseqs++;
-   comments[totseqs] = NULL;
-   p = line + 1; while(*p != ' ' && *p != '\n') p++;
-	l = p - line - 1;
-	if( (seqname[totseqs] = (char *)malloc(l+1)) == NULL)goto nomem;
-	memcpy(seqname[totseqs], line + 1, l); seqname[totseqs][l] = 0;
-	
-	SET_ELEMENT(listname,totseqs,mkChar(seqname[totseqs]));
-       
-	seq[totseqs] = (char *)malloc(lenseqs+1);
-	if(seq[totseqs] == NULL) goto nomem;
-	lseq = 0;
-	while( (i=fgets(line, sizeof(line), in))!= NULL && *i != '>' ) {
-	  l2 = strlen(line);
-	  if( line[l2 - 1] == '\n' ) l2--;
-	  while(l2>0 && line[l2-1]==' ')l2--;
-	  if(lseq + l2 > lenseqs) {
-	    char *temp;
-	    lenseqs += MAXLENSEQ;
-	    temp = (char *)malloc(lenseqs+1);
-	    if(temp == NULL) goto nomem;
-	    memcpy(temp, seq[totseqs], lseq);
-			free(seq[totseqs]);
-			seq[totseqs] = temp;
-	  }
-	  memcpy(seq[totseqs]+lseq, line, l2);
-	  lseq += l2;
-	}
-	seq[totseqs][lseq]='\0';
+  /* Main loop to read line by line the file */
 
-	SET_ELEMENT(listseq,totseqs,mkChar(seq[totseqs]));
-      
-	/* convert all to upper case */
-	p = seq[totseqs] - 1; while( *(++p) != 0 ) *p = toupper(*p);
- }
+  while( i != NULL ){
+    totseqs++;
+    comments[totseqs] = NULL;
+    p = line + 1; 
+    while(*p != '\n')  
+      p++;
+    l = p - line - 1;
 
- SET_ELEMENT(list,0,nombreseq);
- SET_ELEMENT(list,1,listname);
- SET_ELEMENT(list,2,listseq);
+    seqname[totseqs] = (char *) R_alloc(l + 1, sizeof(char));
+
+    memcpy(seqname[totseqs], line + 1, l); 
+    seqname[totseqs][l] = '\0';	
+    SET_ELEMENT(listname, totseqs, mkChar(seqname[totseqs]));
+
+    seq[totseqs] = (char *) R_alloc(lenseqs + 1, sizeof(char));
+    lseq = 0;
+
+    while( (i = fgets(line, sizeof(line), in)) !=  NULL && *i != '>' ) {
+      l2 = strlen(line);
+      if( line[l2 - 1] == '\n' ) l2--;
+      while(l2 > 0 && line[l2 - 1] == ' ')
+        l2--;
+      if(lseq + l2 > lenseqs) {
+        char *temp;
+	lenseqs += MAXLENSEQ;
+	temp = R_alloc(lenseqs + 1, sizeof(char));
+	memcpy(temp, seq[totseqs], lseq);
+	seq[totseqs] = temp;
+      }
+      memcpy(seq[totseqs] + lseq, line, l2);
+      lseq += l2;
+    }
+    seq[totseqs][lseq] = '\0';
+    SET_ELEMENT(listseq, totseqs, mkChar(seq[totseqs]));
+  }
+
+  SET_ELEMENT(list, 0, nombreseq);
+  SET_ELEMENT(list, 1, listname);
+  SET_ELEMENT(list, 2, listseq);
  
- fini:
- fclose(in);
- if(provseq != NULL) free(provseq);
- UNPROTECT(4);
- return list;
- nomem:
- error(" Not enough memory!");
- totseqs = -1;
- goto fini;
+  fclose(in);
+  UNPROTECT(4);
+  return list;
 }
 
 
@@ -768,7 +787,7 @@ SEXP read_clustal_align(SEXP ficname)
   char **seq, **comments, **seqname = NULL;
   
 
-  fname=CHAR(STRING_ELT(ficname,0)); 
+  fname = (char *) CHAR(STRING_ELT(ficname,0)); 
   
   PROTECT(nombreseq=NEW_INTEGER(1));
   PROTECT(list=allocVector(VECSXP,3));
@@ -780,8 +799,8 @@ SEXP read_clustal_align(SEXP ficname)
     return 0;
   }
 
-  fgets(line,sizeof(line),in);
-  if(strncmp(line,"CLUSTAL",7) != 0) { /* skip 1st line with CLUSTAL in it */
+  
+  if(!fgets(line,sizeof(line),in) || strncmp(line,"CLUSTAL",7) != 0) { /* skip 1st line with CLUSTAL in it */
     error("File not in CLUSTAL format!");
     tot_spec = -1; goto fini;
   }
@@ -790,7 +809,11 @@ SEXP read_clustal_align(SEXP ficname)
   do	{
     carac = getc(in);
     if(carac == ' ') {
-      fgets(line,sizeof(line),in);
+      if(!fgets(line,sizeof(line),in)) {
+	    error("Unexpected EOF");
+	    tot_spec = 0;
+	    goto fini;
+       }
       carac = getc(in);
     }
   }
